@@ -19,6 +19,11 @@ import types
 import scipy.special
 _pausemode=True
 import string
+import utils2d
+from functools import wraps
+
+HC=12398.419 #Planck's constant times speed of light, in eV*Angstrom units
+
 
 class SASDict(object):
     """Small Angle Scattering results in a dictionary-like representation.
@@ -513,12 +518,17 @@ class SASDict(object):
         degrees_of_freedom=len(self._dict['q'])-len(p)
         if ier<1 or ier>4:
             raise ValueError('Fitting did not succeed. Reason: %s'%mesg)
-        errors=[ np.sqrt(cov_x[i,i]*chisquare/degrees_of_freedom) for i in range(len(p))]
+            #print "Fitting did not succeed:",mesg
+        if cov_x is None:
+            errors=[np.inf]*len(p)
+            print "Infinite covariance in fitting!"
+        else:
+            errors=[ np.sqrt(cov_x[i,i]*chisquare/degrees_of_freedom) for i in range(len(p))]
         if full_output:
             return p,errors,function(self._dict['q'],*p),chisquare,degrees_of_freedom
         else:
             return p,errors
-    def _fitting_base(self,function,transformdatasettolinear=None, transformparamfromlinear=None,params_initial=None,plotinfo=None):
+    def _fitting_base(self,function,transformdatasettolinear=None, transformparamfromlinear=None,params_initial=None,plotinfo=None,**kwargs):
         """Base fitting function, for internal usage only.
         
          Parameters:
@@ -566,7 +576,7 @@ class SASDict(object):
         elif hasattr(params_initial,'__call__'): #in this case, it is a guessing function
             params_initial=params_initial(self._dict['q'],self._dict['Intensity'])
         # at this point, we have params_initial. Fitting can be carried out.
-        params,errors,curve,chi2,dof=self.fit(function,params_initial,full_output=True)
+        params,errors,curve,chi2,dof=self.fit(function,params_initial,full_output=True,**kwargs)
         #check if plotting was requested.
         if plotinfo is not None:
             if transformdatasettolinear is not None: # if linearization is possible, linearize it.
@@ -753,6 +763,45 @@ class SASDict(object):
                                               [G,R,A,B],
                                               plotinfo)
         return p,e
+    def multigaussfit(self,m=[0],sigma=[1],scaling=[1],plot=True):
+        """Do a multiple Gauss-peaks fit on the dataset
+        
+        Inputs:
+            m: list of expected values
+            sigma: list of variances
+            scaling: list of Intensity scaling values
+            plot: if a plot is requested
+        
+        Outputs: [scaling1,m1,sigma1,scaling2,m2,sigma2,...],[dscaling1,dm1,dsigma1,dscaling2,dm2,dsigma2,...]
+            lists of the fitted parameters and their errors.
+        """
+        nargs=min(len(m),len(sigma),len(scaling))
+        def fitfunction(q,*args):
+            ret=np.zeros(q.shape)
+            for i in xrange(len(args)/3):
+                ret+=args[3*i+0]/np.sqrt(2*np.pi*args[3*i+2]**2)*np.exp(-(q-args[3*i+1])**2/(2*args[3*i+2]**2))
+            return ret
+        paramnames=[]
+        params=[]
+        for i in range(nargs):
+            paramnames.extend(['scaling%d'%(i+1),'m%d'%(i+1),'sigma%d'%(i+1)])
+            params.extend([scaling[i],m[i],sigma[i]])
+        if plot:
+            def legend_addendum(q,I,E,params,errors):
+                return 'q_max*R = %g\n'%(q.max()*params[1])
+            plotinfo={'funcname':'%d Gauss peak(s)'%(nargs),
+                      'paramnames':paramnames,
+                      'otherstringforlegend':legend_addendum}
+        else:
+            plotinfo=None
+        p,e,curve,chi2,dof=self._fitting_base(fitfunction,
+                                              None,
+                                              None,
+                                              params,
+                                              plotinfo)
+        return p,e
+
+
     def trimzoomed(self,inplace=False):
         """Trim dataset according to the current zoom on the last plot.
         
@@ -878,7 +927,25 @@ class SASDict(object):
         #print "returning"
         return listoffits
     bfg=basicfittinggui;
+
+def returnsSASDict(func):
+    """Decorator function for fit functions.
     
+    Usage:
+    
+        @returnsSASDict
+        def powerlaw(q,A,alpha):
+            return A*np.pow(q,alpha)
+        
+        will make powerlaw() to return the results in a SASDict.
+    """
+    @wraps(func)
+    def func1(q,*args,**kwargs):
+        I=func(q,*args,**kwargs)
+        return SASDict(q,I,np.zeros(q.shape))
+    return func1
+
+
 class SASTransform(object):
     def __init__(self):
         pass
@@ -967,39 +1034,172 @@ TransformSemilogX=SASTransformLogLog(True,False)
 TransformSemilogY=SASTransformLogLog(False,True)
 TransformLinLin=SASTransformLogLog(False,False)
 
-
 class SASImage(object):
     def __init__(self,A,Aerr=None,param=None,mask=None):
         self._A=A
         self._Aerr=Aerr
         self._param=param
         self._mask=mask
+        self._q=None
+    def __del__(self):
+        object.__delattr__(self,'_A')
+        object.__delattr__(self,'_Aerr')
+        object.__delattr__(self,'_mask')
+        object.__delattr__(self,'_q')
     def __getitem__(self,item):
         try:
             return self._param[item]
         except KeyError:
             raise
-    def _getIntensity(self):
-        return np.array(self._A)
-    def _setIntensity(self,A1):
-        if (self._A is None) or (self._A.shape==A1.shape):
-            self._A=np.array(A1)
+    def __getattr__(self,key):
+        if key in ['A','Aerr','mask','param']:
+            key='_%s'%key
+            x=object.__getattribute__(self,key)
+            if type(x)==np.ndarray:
+                return np.array(x)
+            else:
+                return x
+        if key=='q':
+            return object.__getattribute__(self,'_getq').__call__()
+        try:
+            return object.__getattribute__(self,key)
+        except AttributeError:
+            if not key.startswith('_'):
+                return object.__getattribute__(self,'_%s'%key)
+            raise
+    def __setattr__(self,key,value):
+        if key in ['A','Aerr', 'mask']:
+            key='_%s'%key
+            value=np.array(value)
+            if (value.shape==self.A.shape):
+                object.__setattr__(self,key,value)
+            elif key=='A':
+                object.__setattr__(self,key,value)
+                object.__setattr__(self,'Aerr',None)
+                object.__setattr__(self,'mask',None)
+            else:
+                raise ValueError('Cannot broadcast objects to a single shape!')
         else:
-            raise ValueError('Incompatible shape for Intensity matrix!')
-    def _getError(self):
-        return np.array(self._Aerr)
-    def _setError(self,A1):
-        if (self._Aerr is None) or (self._Aerr.shape==A1.shape):
-            self._Aerr=np.array(A1)
-        else:
-            raise ValueError('Incompatible shape for Error matrix!')
+            return object.__setattr__(self,key,value)
     def _getq(self):
-        raise NotImplementedError
-    Intensity=property(fget=_getIntensity,fset=_setIntensity,doc='Two-dimensional scattering intensity')
-    Error=property(fget=_getError,fset=_setError,doc='Absolute error of two-dimensional scattering intensity')
-    q=property(fget=_getq,doc='length of momentum transfer vector')
+        if not all([x in self._param for x in ['BeamPosX','BeamPosY','Dist','EnergyCalibrated','PixelSize']]):
+            raise ValueError('Cannot calculate q! Some parameters are missing.')
+        if self._q is None:
+            self._q=utils2d.calculateqmatrix(self.A.astype(np.uint8),self['Dist'],self['EnergyCalibrated'],self['PixelSize'],self['BeamPosX'],self['BeamPosY'])
+        return self._q
+    def _qscalepossible(self):
+        res=False
+        if self._param is not None:
+            if all([x in self._param.keys() for x in ['BeamPosX','BeamPosY','Dist','EnergyCalibrated','PixelSize']]):
+                res=True
+        return res
+    def plot(self,maxval=np.inf,minval=-np.inf,qs=[],showqscale=True,blackinvalid=False,crosshair=True,zscaling='log'):
+        """Plots the scattering image in a 2D coloured plot
+        
+        Inputs:
+            maxval: upper saturation of the colour scale. +inf (default): auto
+            minval: lower saturation of the colour scale. -inf (default): auto
+            qs: q-values for which concentric circles will be drawn. An empty list by default.
+            showqscale: show q-scale on both the horizontal and vertical axes. True by default.
+            blackinvalid: True if nonpositive and nonfinite pixels should be
+                blacked out. False by default.
+            crosshair: True if you want to draw a beam-center testing cross-hair.
+                False by default.
+            zscaling: 'log' or 'linear' (color scaling)
+        """
+        tmp=self._A.copy()
+        tmp[tmp>maxval]=max(tmp[tmp<=maxval])
+        tmp[tmp<minval]=min(tmp[tmp>=minval])
+        nonpos=(tmp<=0)
+        invalid=-np.isfinite(tmp)
+        if zscaling.upper().startswith('LOG'):
+            tmp[nonpos]=tmp[tmp>0].min()
+            tmp=np.log(tmp);
+        tmp[invalid]=tmp[-invalid].min();
+        if showqscale:
+            if not self._qscalepossible():
+                raise ValueError('field *param* not defined in SASImage and q-scaling requested in SASImage.plot().')
+            # x: row direction (vertical on plot)
+            xmin=0-(self._param['BeamPosX']-1)*self._param['PixelSize']
+            xmax=(tmp.shape[0]-(self._param['BeamPosX']-1))*self._param['PixelSize']
+            ymin=0-(self._param['BeamPosY']-1)*self._param['PixelSize']
+            ymax=(tmp.shape[1]-(self._param['BeamPosY']-1))*self._param['PixelSize']
+            qxmin=4*np.pi*np.sin(0.5*np.arctan(xmin/self._param['Dist']))*self._param['EnergyCalibrated']/float(HC)
+            qxmax=4*np.pi*np.sin(0.5*np.arctan(xmax/self._param['Dist']))*self._param['EnergyCalibrated']/float(HC)
+            qymin=4*np.pi*np.sin(0.5*np.arctan(ymin/self._param['Dist']))*self._param['EnergyCalibrated']/float(HC)
+            qymax=4*np.pi*np.sin(0.5*np.arctan(ymax/self._param['Dist']))*self._param['EnergyCalibrated']/float(HC)
+            extent=[qymin,qymax,qxmax,qxmin]
+            bcrow=0
+            bccol=0
+        else: #not q-scaling requested
+            extent=None
+            if (self._param is not None) and ('BeamPosX' in self._param.keys()) and ('BeamPosY' in self._param.keys()):
+                bcrow=self._param['BeamPosX']
+                bccol=self._param['BeamPosY']
+            else:
+                bcrow=None
+                bccol=None
+            extent=[1,tmp.shape[0],1,tmp.shape[1]]
+        pylab.imshow(tmp,extent=extent,interpolation='nearest');
+        if blackinvalid:
+            black=np.zeros((tmp.shape[0],tmp.shape[1],4))
+            black[:,:,3][nonpos|invalid]=1
+            pylab.imshow(black,extent=extent,interpolation='nearest')
+            del black;
+        if self._mask is not None:
+            white=np.ones((self._mask.shape[0],self._mask.shape[1],4))
+            white[:,:,3]=np.array(1-self._mask).astype('float')*0.7
+            pylab.imshow(white,extent=extent,interpolation='nearest')
+            del white;
+        for q in qs:
+            a=pylab.gca().axis()
+            pylab.plot(q*np.cos(np.linspace(0,2*np.pi,2000)),
+                       q*np.sin(np.linspace(0,2*np.pi,2000)),
+                       color='white',linewidth=3)
+            pylab.gca().axis(a)
+        if (self._param is not None) and ('Title' in self._param.keys()) and ('FSN' in self._param.keys()):
+            pylab.title("#%d: %s" % (self._param['FSN'], self._param['Title']))
+        if crosshair and (bcrow is not None):
+            a=pylab.gca().axis()
+            pylab.plot([extent[0],extent[1]],[bcrow,bcrow],'-',color='white')
+            pylab.plot([bccol,bccol],[extent[2],extent[3]],'-',color='white')
+            pylab.gca().axis(a)
+        del tmp;
+    def radint(self,q=None,returnmask=False):
+        q1,I1,err1,Area1,mask1=utils2d.radintC(self._A,self._Aerr,
+                                       self._param['EnergyCalibrated'],
+                                       self._param['Dist'],
+                                       self._param['PixelSize'],
+                                       self._param['BeamPosX'],
+                                       self._param['BeamPosY'],
+                                       self._mask,q,returnavgq=True,
+                                       returnmask=returnmask)
+        return SASDict(q=q1,Intensity=I1,Error=err1,Area=Area1)
+    @staticmethod
+    def loadmatfile(matfile):
+        a=scipy.io.loadmat(matfile)
+        if ('Intensity' in a.keys()) and ('Error' in a.keys()):
+            return SASImage(a['Intensity'],a['Error'])
+        else:
+            vars=[x for x in a.keys() if not ((x.startswith('__') or x.endswith('__')))]
+            if len(vars)==1:
+                return SASImage(a[vars[0]])
+            else:
+                raise IOError('Invalid mat file %s'%matfile)
+    @staticmethod
+    def loadnumpyfile(numpyfile):
+        a=np.load(numpyfile)
+        if ('Intensity' in a.files) and ('Error' in a.files):
+            return SASImage(a['Intensity'],a['Error'])
+        else:
+            vars=[x for x in a.files if not ((x.startswith('__') or (x.endswith('__'))))]
+            if len(vars)==1:
+                return SASImage(a[vars[0]])
+            else:
+                raise IOError('Invalid numpy file %s'%numpyfile)
+        
+        
     
-
 def trimq(data,qmin=-np.inf,qmax=np.inf):
     """Trim the 1D scattering data to a given q-range
     
